From: "H. Peter Anvin" Subject: Re: [PATCH RFC V2 3/5] jump_label: if a key has already been initialized, don't nop it out Date: Thu, 06 Oct 2011 11:29:25 -0700 Message-ID: <4E8DF385.3070009@zytor.com> References: <477dead9647029012f93c651f2892ed0e86b89e7.1317506051.git.jeremy.fitzhardinge@citrix.com> <20111003150205.GB2462@redhat.com> <4E89E28C.7010700@goop.org> <20111004141011.GA2520@redhat.com> <4E8B3489.60902@zytor.com> <4E8CF348.4080405@goop.org> <4E8CF385.2080804@zytor.com> <4E8DEB19.1050509@goop.org> <20111006181055.GA2505@redhat.com> <1317925615.4729.14.camel@gandalf.stny.rr.com> Mime-Version: 1.0 Content-Type: text/plain; charset=UTF-8 Content-Transfer-Encoding: 7bit Return-path: In-Reply-To: <1317925615.4729.14.camel@gandalf.stny.rr.com> List-Unsubscribe: , List-Post: List-Help: List-Subscribe: , Sender: xen-devel-bounces@lists.xensource.com Errors-To: xen-devel-bounces@lists.xensource.com To: Steven Rostedt Cc: Jeremy Fitzhardinge , the arch/x86 maintainers , Jeremy Fitzhardinge , David Daney , peterz@infradead.org, Jason Baron , Jan Glauber , Linux Kernel Mailing List , rth@redhat.com, Michael Ellerman , Xen Devel , "David S. Miller" List-Id: xen-devel@lists.xenproject.org On 10/06/2011 11:26 AM, Steven Rostedt wrote: > On Thu, 2011-10-06 at 14:10 -0400, Jason Baron wrote: > >>> Looks like jmp2 is about 5% faster than jmp5 on Sandybridge with this >>> benchmark. >>> >>> But insignificant difference on Nehalem. >>> >>> J >> >> It would be cool if we could make the total width 2-bytes, when >> possible. It might be possible by making the initial 'JUMP_LABEL_INITIAL_NOP' >> as a 'jmp' to the 'l_yes' label. And then patching that with a no-op at boot >> time or link time - letting the compiler pick the width. In that way we could >> get the optimal width... > > Why not just do it? > > jump_label is encapsulated in arch_static_branch() which on x86 looks > like: > > static __always_inline bool arch_static_branch(struct jump_label_key *key) > { > asm goto("1:" > JUMP_LABEL_INITIAL_NOP > ".pushsection __jump_table, \"aw\" \n\t" > _ASM_ALIGN "\n\t" > _ASM_PTR "1b, %l[l_yes], %c0 \n\t" > ".popsection \n\t" > : : "i" (key) : : l_yes); > return false; > l_yes: > return true; > } > > > That jmp to l_yes should easily be a two byte jump. > > If not I'm sure it would be easy to catch it before modifying the code. > And then complain real loudly about it. > The important thing is that it requires the build-time elimination of jumps. It's just work. -hpa -- H. Peter Anvin, Intel Open Source Technology Center I work for Intel. I don't speak on their behalf.